/**
 * Vue Number Format 3.16.0
 * (c) 2021-2023 Dipak Sarkar <hello@dipaksarkar.in> (https://dipaksarkar.in/)
 * @license MIT
 */
import { defineComponent, ref, computed, resolveDirective, withDirectives, openBlock, createElementBlock } from 'vue';

var options$1 = {
    prefix: '',
    suffix: '',
    separator: ',',
    decimal: '.',
    precision: 2,
    minimumFractionDigits: null,
    prefill: true,
    reverseFill: false,
    min: null,
    max: null,
    nullValue: ''
};

function between(min, n, max) {
    return Math.max(min, Math.min(n, max));
}
function fixed(precision) {
    return between(0, precision, 20);
}
function toFixed(numbers, precision) {
    const exp = Math.pow(10, precision);
    const float = parseFloat(numbers) / exp || 0;
    return float.toFixed(fixed(precision));
}
class NumberFormat {
    options;
    input;
    number;
    isClean;
    preSurRegExp;
    numberRegExp;
    cleanRegExp;
    negativeRegExp;
    constructor(config) {
        this.options = Object.assign(options$1, config);
        this.input = '';
        this.number = '';
        this.isClean = true;
        const { prefix, suffix, decimal } = this.options;
        this.preSurRegExp = new RegExp(`${prefix}|${suffix}`, 'g');
        this.numberRegExp = new RegExp(`[^0-9\\${decimal}]+`, 'gi');
        this.cleanRegExp = new RegExp('[^0-9]+', 'gi');
        this.negativeRegExp = new RegExp('[^0-9\\-]+', 'gi');
    }
    isNull() {
        if (this.isClean) {
            return !this.numberOnly(this.input, this.cleanRegExp);
        }
        return !this.numberOnly(this.input, this.negativeRegExp);
    }
    clean(clean = false) {
        this.isClean = clean;
        return this;
    }
    sign() {
        if (this.input === null || this.input === undefined) {
            return '';
        }
        const hasMinus = this.input.toString().indexOf('-') >= 0;
        if (this.isClean) {
            return hasMinus && this.realNumber() > 0 ? '-' : '';
        }
        return hasMinus ? '-' : '';
    }
    toNumber(str) {
        return Number(str);
    }
    numberOnly(str, regExp) {
        return str?.toString().replace(regExp, '');
    }
    isNegative() {
        return this.sign() === '-';
    }
    numbers() {
        if (this.options.reverseFill) {
            this.number = toFixed(this.numberOnly(this.input, /\D+/g), this.options.precision).replace('.', this.options.decimal);
        }
        else if (typeof this.input === 'number') {
            this.number = this.parts(this.input.toString().replace('-', ''), '.').join(this.options.decimal);
        }
        else {
            const input = this.input.replace(this.preSurRegExp, '');
            this.number = this.numberOnly(input, this.numberRegExp);
            this.number = this.parts(this.number).join(this.options.decimal);
        }
        return this.number;
    }
    realNumber() {
        return parseFloat(this.numbers().toString().replace(this.options.decimal, '.'));
    }
    parts(num, separator) {
        const decimal = separator || this.options.decimal;
        let parts = num.toString().split(decimal);
        if (parts.length > 1) {
            parts[0] = this.toNumber(parts[0]) || 0;
            parts[1] = parts.slice(1, parts.length).join('');
            parts = parts.slice(0, 2);
        }
        if (this.isClean) {
            const newNumber = this.toNumber(parts.join('.')).toFixed(this.options.precision);
            const cleanNumber = this.toNumber(newNumber);
            const minimumDigits = cleanNumber.toFixed(this.options.minimumFractionDigits);
            const hasMinFraction = this.options.minimumFractionDigits >= 0 && cleanNumber.toString().length < minimumDigits.length;
            if (hasMinFraction) {
                parts = minimumDigits.toString().split('.');
            }
            else {
                parts = cleanNumber.toString().split('.');
            }
        }
        return parts.slice(0, 2);
    }
    addSeparator() {
        const parts = this.numbers().split(this.options.decimal);
        parts[0] = parts[0].toString().replace(/(\d)(?=(?:\d{3})+\b)/gm, `$1${this.options.separator}`);
        return parts.join(this.options.decimal);
    }
    format(input) {
        this.input = input;
        if (this.isNull() && !this.options.reverseFill) {
            return this.options.nullValue;
        }
        return this.sign() + this.options.prefix + this.addSeparator() + this.options.suffix;
    }
    unformat(input) {
        this.input = input;
        if (this.isNull() && !this.options.reverseFill) {
            return this.options.nullValue;
        }
        return this.sign() + this.realNumber();
    }
}

const MINUS = '-';
function cloneDeep(data) {
    return JSON.parse(JSON.stringify(data));
}
function FacadeEvent(event) {
    return new CustomEvent(event, {
        bubbles: true,
        cancelable: true,
        detail: { facade: true }
    });
}
function getInputElement(el) {
    const inputElement = el instanceof HTMLInputElement ? el : el.querySelector('input');
    if (!inputElement) {
        throw new Error('facade directive requires an input element');
    }
    return inputElement;
}
function updateCursor(el, position) {
    const setSelectionRange = () => {
        el.setSelectionRange(position, position);
    };
    setSelectionRange();
    setTimeout(setSelectionRange(), 1);
}
function updateValue(el, vnode, { emit = true, force = false, clean = false } = {}) {
    const { options, oldValue } = el;
    const currentValue = vnode?.props?.value || el.value;
    if (force || oldValue !== currentValue) {
        const number = new NumberFormat(options).clean(clean && !options.reverseFill);
        let masked = number.format(currentValue);
        let unmasked = number.clean(options && !options.reverseFill).unformat(currentValue);
        if (clean) {
            if (Number(options.max) === options.max && Number(unmasked) > options.max) {
                masked = number.format(options.max);
                unmasked = number.unformat(options.max);
            }
            else if (Number(options.min) === options.min && Number(unmasked) < options.min) {
                masked = number.format(options.min);
                unmasked = number.unformat(options.min);
            }
        }
        el.masked = masked;
        el.unmaskedValue = unmasked;
        if (el.value !== masked) {
            el.value = masked;
        }
        return emit && el.dispatchEvent(FacadeEvent('input'));
    }
}
function inputHandler(event) {
    const { target, detail } = event;
    if (detail?.facade) {
        return false;
    }
    const { oldValue, options, masked } = target;
    event.stopPropagation();
    let positionFromEnd = target.value.length;
    if (target.selectionEnd) {
        positionFromEnd = target.value.length - target.selectionEnd;
    }
    updateValue(target, null, { emit: false });
    if (options.suffix) {
        positionFromEnd = Math.max(positionFromEnd, options.suffix.length);
    }
    positionFromEnd = target.value.length - positionFromEnd;
    if (options.prefix) {
        positionFromEnd = Math.max(positionFromEnd, options.prefix.length);
    }
    updateCursor(target, positionFromEnd);
    if (oldValue !== target.value) {
        target.oldValue = masked;
        target.dispatchEvent(FacadeEvent('input'));
    }
}
function blurHandler(event) {
    const { target, detail } = event;
    if (detail?.facade) {
        return false;
    }
    const { oldValue, masked } = target;
    updateValue(target, null, { force: true, emit: true, clean: true });
    if (oldValue !== target.value) {
        target.oldValue = masked;
        target.dispatchEvent(FacadeEvent('change'));
    }
}
function keydownHandler(event, el) {
    const { options } = el;
    const { prefix, suffix, decimal, min, separator } = options;
    const { key } = event;
    const regExp = new RegExp(`${prefix}|${suffix}`, 'g');
    const newValue = el.value.replace(regExp, '');
    const canNegativeInput = min === undefined || Number(min) < 0 || Number(min) !== min;
    if (key === decimal && newValue.includes(decimal)) {
        event.preventDefault();
    }
    else if (key === MINUS && !canNegativeInput) {
        event.preventDefault();
    }
    else if (key === 'Backspace') {
        const selectionEnd = el.selectionEnd || 0;
        const character = el.value.slice(selectionEnd - 1, selectionEnd);
        const replace = el.value.slice(selectionEnd - 2, selectionEnd);
        let positionFromEnd = el.value.length - selectionEnd;
        if ([prefix, MINUS, separator].includes(character)) {
            event.preventDefault();
            if (character === separator) {
                el.value = el.value.replace(replace, '');
            }
            else {
                el.value = el.value.replace(new RegExp(`[${prefix}${MINUS}]`, 'g'), '');
            }
            positionFromEnd = Math.max(positionFromEnd, suffix.length);
            positionFromEnd = el.value.length - positionFromEnd;
            positionFromEnd = Math.max(positionFromEnd, prefix.length);
            updateCursor(el, positionFromEnd);
            el.dispatchEvent(new Event('input'));
        }
    }
}

var directive = {
    beforeMount: (el, { value, modifiers }, vnode) => {
        el = getInputElement(el);
        const options = Object.assign(cloneDeep(options$1), value, modifiers);
        el.options = options;
        updateValue(el, vnode, { force: options.prefill, clean: true });
    },
    mounted: (el) => {
        el = getInputElement(el);
        const handlerOwner = el.parentElement || el;
        const oninput = (e) => {
            if (e.target !== el) {
                return;
            }
            inputHandler(e);
        };
        handlerOwner.addEventListener('input', oninput, true);
        el.onblur = (e) => blurHandler(e);
        el.onkeydown = (e) => keydownHandler(e, el);
        el.cleanup = () => handlerOwner.removeEventListener('input', oninput, true);
    },
    updated: (el, { value, oldValue, modifiers }, vnode) => {
        el = getInputElement(el);
        const options = el.options;
        el.options = Object.assign(options, value, modifiers);
        if (value !== oldValue) {
            updateValue(el, vnode, { force: true, clean: true });
        }
        else {
            updateValue(el, vnode);
        }
    },
    unmounted: (el) => {
        getInputElement(el).cleanup();
    }
};

const options = cloneDeep(options$1);
var script = defineComponent({
    name: 'VueNumber',
    directives: {
        number: directive
    },
    props: {
        modelValue: {
            type: [String, Number],
            required: true
        },
        nullValue: {
            type: [Number, String],
            required: false,
            default: options.nullValue
        },
        masked: {
            type: Boolean,
            default: false
        },
        reverseFill: {
            type: Boolean,
            default: options.reverseFill
        },
        prefill: {
            type: Boolean,
            default: options.prefill
        },
        precision: {
            type: Number,
            default: () => options.precision
        },
        minimumFractionDigits: {
            type: Number,
            default: () => options.minimumFractionDigits
        },
        decimal: {
            type: String,
            default: () => options.decimal
        },
        min: {
            type: Number,
            default: () => options.min
        },
        max: {
            type: Number,
            default: () => options.max
        },
        separator: {
            type: String,
            default: () => options.separator
        },
        prefix: {
            type: String,
            default: () => options.prefix
        },
        suffix: {
            type: String,
            default: () => options.suffix
        }
    },
    emits: ['update:model-value', 'input:model-value'],
    setup(props, { emit }) {
        const maskedValue = ref(props.modelValue);
        const unmaskedValue = ref('');
        const config = computed(() => ({ ...props }));
        const emittedValue = computed(() => {
            return props.masked ? maskedValue.value : unmaskedValue.value;
        });
        const input = (event) => {
            const { target } = event;
            maskedValue.value = target.value;
            unmaskedValue.value = target.unmaskedValue;
            emit('input:model-value', emittedValue);
        };
        const change = () => {
            emit('update:model-value', emittedValue);
        };
        return {
            config,
            maskedValue,
            input,
            change
        };
    }
});

const _hoisted_1 = ["value"];
function render(_ctx, _cache, $props, $setup, $data, $options) {
  const _directive_number = resolveDirective("number");
  return withDirectives((openBlock(), createElementBlock("input", {
    type: "text",
    autocomplete: "off",
    value: _ctx.maskedValue,
    class: "v-number vue-number-format",
    onChange: _cache[0] || (_cache[0] = (...args) => (_ctx.change && _ctx.change(...args))),
    onInput: _cache[1] || (_cache[1] = (...args) => (_ctx.input && _ctx.input(...args)))
  }, null, 40 , _hoisted_1)), [
    [_directive_number, _ctx.config]
  ])
}

script.render = render;
script.__file = "src/component.vue";

const VueNumberFormat = {
    install(app, config) {
        if (config) {
            Object.assign(options$1, config);
        }
        app.directive('number', directive);
        app.component('VueNumber', script);
    }
};

export { NumberFormat, script as component, VueNumberFormat as default, directive, options$1 as options };
